//!
//!
use super::{models, schema::api_kgon_betting_history};
use beteran_common_rust as bcr;
use diesel::prelude::*;
use diesel::result::Error;

///
pub struct Repository {}

impl std::fmt::Debug for Repository {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Repository of api_kgon_betting_history")
      .finish()
  }
}

impl Default for Repository {
  fn default() -> Self {
    Self::new()
  }
}

impl Repository {
  ///
  pub fn new() -> Repository {
    Repository {}
  }

  ///
  pub fn insert(
    &self,
    conn: &diesel::PgConnection,
    new_betting_history: &models::NewBettingHistory,
  ) -> Result<(), Error> {
    diesel::insert_into(api_kgon_betting_history::table)
      .values(new_betting_history)
      .execute(conn)?;

    Ok(())
  }

  ///
  pub fn inserts(
    &self,
    conn: &diesel::PgConnection,
    new_betting_history: &Vec<models::NewBettingHistory>,
  ) -> Result<(), Error> {
    use api_kgon_betting_history::dsl;

    diesel::insert_into(api_kgon_betting_history::table)
      .values(new_betting_history)
      .on_conflict(dsl::id)
      .do_nothing()
      .execute(conn)?;

    Ok(())
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: String,
  ) -> Result<Option<models::BettingHistory>, Error> {
    match api_kgon_betting_history::table
      .find(id)
      .first::<models::BettingHistory>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<i64, Error> {
    let mut q = api_kgon_betting_history::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.vendor_id {
        q = q.filter(api_kgon_betting_history::dsl::vendor_id.eq(sp));
      }
      if let Some(sp) = &s.vendor_name_like {
        q = q.filter(api_kgon_betting_history::dsl::vendor_name.like(sp));
      }
      if let Some(sp) = s.game_id {
        q = q.filter(api_kgon_betting_history::dsl::game_id.eq(sp));
      }
      if let Some(sp) = &s.game_name_like {
        q = q.filter(api_kgon_betting_history::dsl::game_name.like(sp));
      }
      if let Some(sp) = &s.game_category_like {
        q = q.filter(api_kgon_betting_history::dsl::game_category.like(sp));
      }
      if let Some(sp) = &s.game_type_like {
        q = q.filter(api_kgon_betting_history::dsl::game_type.like(sp));
      }
      if let Some(sp) = &s.currency_like {
        q = q.filter(api_kgon_betting_history::dsl::currency.like(sp));
      }
      if let Some(sp) = &s.key_like {
        q = q.filter(api_kgon_betting_history::dsl::key.like(sp));
      }
      if let Some(sp) = &s.ref_id_like {
        q = q.filter(api_kgon_betting_history::dsl::ref_id.like(sp));
      }
      if let Some(sp) = &s.o_ref_id_like {
        q = q.filter(api_kgon_betting_history::dsl::o_ref_id.like(sp));
      }
      if let Some(sp) = &s.group_key_like {
        q = q.filter(api_kgon_betting_history::dsl::group_key.like(sp));
      }
      if let Some(sp) = s.is_bonus {
        q = q.filter(api_kgon_betting_history::dsl::is_bonus.eq(sp));
      }
      if let Some(sp) = s.is_promo {
        q = q.filter(api_kgon_betting_history::dsl::is_promo.eq(sp));
      }
      if let Some(sp) = s.is_jackpot {
        q = q.filter(api_kgon_betting_history::dsl::is_jackpot.eq(sp));
      }
      if let Some(sp) = &s.site_username_like {
        q = q.filter(api_kgon_betting_history::dsl::site_username.like(sp));
      }
      if let Some(sp) = &s.betting_type_like {
        q = q.filter(api_kgon_betting_history::dsl::betting_type.like(sp));
      }
      if let Some(sp) = &s.category_like {
        q = q.filter(api_kgon_betting_history::dsl::category.like(sp));
      }
    }

    q.count().get_result(conn)
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &models::FindAll,
  ) -> Result<Vec<models::BettingHistory>, Error> {
    let mut q = api_kgon_betting_history::table.into_boxed();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.vendor_id {
        q = q.filter(api_kgon_betting_history::dsl::vendor_id.eq(sp));
      }
      if let Some(sp) = &s.vendor_name_like {
        q = q.filter(api_kgon_betting_history::dsl::vendor_name.like(sp));
      }
      if let Some(sp) = s.game_id {
        q = q.filter(api_kgon_betting_history::dsl::game_id.eq(sp));
      }
      if let Some(sp) = &s.game_name_like {
        q = q.filter(api_kgon_betting_history::dsl::game_name.like(sp));
      }
      if let Some(sp) = &s.game_category_like {
        q = q.filter(api_kgon_betting_history::dsl::game_category.like(sp));
      }
      if let Some(sp) = &s.game_type_like {
        q = q.filter(api_kgon_betting_history::dsl::game_type.like(sp));
      }
      if let Some(sp) = &s.currency_like {
        q = q.filter(api_kgon_betting_history::dsl::currency.like(sp));
      }
      if let Some(sp) = &s.key_like {
        q = q.filter(api_kgon_betting_history::dsl::key.like(sp));
      }
      if let Some(sp) = &s.ref_id_like {
        q = q.filter(api_kgon_betting_history::dsl::ref_id.like(sp));
      }
      if let Some(sp) = &s.o_ref_id_like {
        q = q.filter(api_kgon_betting_history::dsl::o_ref_id.like(sp));
      }
      if let Some(sp) = &s.group_key_like {
        q = q.filter(api_kgon_betting_history::dsl::group_key.like(sp));
      }
      if let Some(sp) = s.is_bonus {
        q = q.filter(api_kgon_betting_history::dsl::is_bonus.eq(sp));
      }
      if let Some(sp) = s.is_promo {
        q = q.filter(api_kgon_betting_history::dsl::is_promo.eq(sp));
      }
      if let Some(sp) = s.is_jackpot {
        q = q.filter(api_kgon_betting_history::dsl::is_jackpot.eq(sp));
      }
      if let Some(sp) = &s.site_username_like {
        q = q.filter(api_kgon_betting_history::dsl::site_username.like(sp));
      }
      if let Some(sp) = &s.betting_type_like {
        q = q.filter(api_kgon_betting_history::dsl::betting_type.like(sp));
      }
      if let Some(sp) = &s.category_like {
        q = q.filter(api_kgon_betting_history::dsl::category.like(sp));
      }
    }

    if let Some(p) = &find_all.pagination {
      let page = p.page.unwrap_or(1);

      if let Some(page_size) = p.page_size {
        q = q.offset(((page - 1) * page_size) as i64);
        q = q.limit(page_size as i64);
      }
    }
    if let Some(orderbys) = &find_all.sorts {
      for s in orderbys {
        match s {
          bcr::pagination::Sort::ASC(property) => match property.as_str() {
            "id" => {
              q = q.order_by(api_kgon_betting_history::id.asc());
            }
            "vendor_id" => {
              q = q.order_by(api_kgon_betting_history::vendor_id.asc());
            }
            "vendor_name" => {
              q = q.order_by(api_kgon_betting_history::vendor_name.asc());
            }
            "game_id" => {
              q = q.order_by(api_kgon_betting_history::game_id.asc());
            }
            "game_name" => {
              q = q.order_by(api_kgon_betting_history::game_name.asc());
            }
            "game_category" => {
              q = q.order_by(api_kgon_betting_history::game_category.asc());
            }
            "game_type" => {
              q = q.order_by(api_kgon_betting_history::game_type.asc());
            }
            "currency" => {
              q = q.order_by(api_kgon_betting_history::currency.asc());
            }
            "cash" => {
              q = q.order_by(api_kgon_betting_history::cash.asc());
            }
            "before_cash" => {
              q = q.order_by(api_kgon_betting_history::before_cash.asc());
            }
            "after_cash" => {
              q = q.order_by(api_kgon_betting_history::after_cash.asc());
            }
            "key" => {
              q = q.order_by(api_kgon_betting_history::key.asc());
            }
            "ref_id" => {
              q = q.order_by(api_kgon_betting_history::ref_id.asc());
            }
            "o_ref_id" => {
              q = q.order_by(api_kgon_betting_history::o_ref_id.asc());
            }
            "group_key" => {
              q = q.order_by(api_kgon_betting_history::group_key.asc());
            }
            "is_bonus" => {
              q = q.order_by(api_kgon_betting_history::is_bonus.asc());
            }
            "is_promo" => {
              q = q.order_by(api_kgon_betting_history::is_promo.asc());
            }
            "is_jackpot" => {
              q = q.order_by(api_kgon_betting_history::is_jackpot.asc());
            }
            "site_username" => {
              q = q.order_by(api_kgon_betting_history::site_username.asc());
            }
            "betting_type" => {
              q = q.order_by(api_kgon_betting_history::betting_type.asc());
            }
            "category" => {
              q = q.order_by(api_kgon_betting_history::category.asc());
            }
            "created_at" => {
              q = q.order_by(api_kgon_betting_history::created_at.asc());
            }
            "utc_created_at" => {
              q = q.order_by(api_kgon_betting_history::utc_created_at.asc());
            }
            _ => {}
          },
          bcr::pagination::Sort::DESC(property) => match property.as_str() {
            "id" => {
              q = q.order_by(api_kgon_betting_history::id.desc());
            }
            "vendor_id" => {
              q = q.order_by(api_kgon_betting_history::vendor_id.desc());
            }
            "vendor_name" => {
              q = q.order_by(api_kgon_betting_history::vendor_name.desc());
            }
            "game_id" => {
              q = q.order_by(api_kgon_betting_history::game_id.desc());
            }
            "game_name" => {
              q = q.order_by(api_kgon_betting_history::game_name.desc());
            }
            "game_category" => {
              q = q.order_by(api_kgon_betting_history::game_category.desc());
            }
            "game_type" => {
              q = q.order_by(api_kgon_betting_history::game_type.desc());
            }
            "currency" => {
              q = q.order_by(api_kgon_betting_history::currency.desc());
            }
            "cash" => {
              q = q.order_by(api_kgon_betting_history::cash.desc());
            }
            "before_cash" => {
              q = q.order_by(api_kgon_betting_history::before_cash.desc());
            }
            "after_cash" => {
              q = q.order_by(api_kgon_betting_history::after_cash.desc());
            }
            "key" => {
              q = q.order_by(api_kgon_betting_history::key.desc());
            }
            "ref_id" => {
              q = q.order_by(api_kgon_betting_history::ref_id.desc());
            }
            "o_ref_id" => {
              q = q.order_by(api_kgon_betting_history::o_ref_id.desc());
            }
            "group_key" => {
              q = q.order_by(api_kgon_betting_history::group_key.desc());
            }
            "is_bonus" => {
              q = q.order_by(api_kgon_betting_history::is_bonus.desc());
            }
            "is_promo" => {
              q = q.order_by(api_kgon_betting_history::is_promo.desc());
            }
            "is_jackpot" => {
              q = q.order_by(api_kgon_betting_history::is_jackpot.desc());
            }
            "site_username" => {
              q = q.order_by(api_kgon_betting_history::site_username.desc());
            }
            "betting_type" => {
              q = q.order_by(api_kgon_betting_history::betting_type.desc());
            }
            "category" => {
              q = q.order_by(api_kgon_betting_history::category.desc());
            }
            "created_at" => {
              q = q.order_by(api_kgon_betting_history::created_at.desc());
            }
            "utc_created_at" => {
              q = q.order_by(api_kgon_betting_history::utc_created_at.desc());
            }
            _ => {}
          },
        };
      }
    }

    q.load::<models::BettingHistory>(conn)
  }
}
